-- name=int_button_collectibles
--******************************************************************************************
function public.Init ( pos_z )
  
  ObjAttach( "int_button_collectibles", interface.originhub );
  MsgSubscribe( Command_Interface_UpdateTextures, private.WideScreenUpdate );

  private.DIRECTION =  -1;

  private.MOUSE_OVER_ANIM_TIME = 0.3;

  private.MOVE_ANIM_POS_X_BEG =   -240;
  private.MOVE_ANIM_POS_X_END =   215;
  private.MOVE_ANIM_TIME      =  0.6;

  private.show  = false; --на экране или нет
  private.timer = 3; --время таймера
  private.count_collect_item = 0;

  private.WideScreenUpdate();

  ObjMultiSet( {
  { "int_button_collectibles", { pos_z = pos_z } },
  { "obj_int_button_collectibles_root", { visible = false } },
  { "obj_int_button_collectibles", { pos_x = private.MOVE_ANIM_POS_X_BEG } },
  { "obj_int_button_collectibles_button", { event_mdown  = private.GoCollectibles, 
                                            event_menter = private.MouseEnter,
                                            event_mleave = private.MouseLeave } }, 
  } );

end;
--******************************************************************************************
function public.Destroy ()

  MsgUnsubscribe( Command_Interface_UpdateTextures, private.WideScreenUpdate );

end;
--******************************************************************************************
function public.Show ( number, name )

  private.count_collect_item = number;
  ObjSet( "obj_int_button_collectibles_root", { active = true, visible = true, input = true } );
  private.show = true;
  private.ShowAnim( "show", name );  
  
end;
--******************************************************************************************
function public.Hide ()

  ObjSet( "obj_int_button_collectibles_root", { input = false } );
  private.show = false;
  private.HideAnim( "hide" );

end;
--******************************************************************************************
function private.WideScreenUpdate ()

  local direction = private.DIRECTION or 0;

  if ( direction ~= 0 and ObjGet( "int_button_collectibles" ) ) then

    local offset = 0.5 * ( GetAppWidth() - 1024 );
    local position =  0;

    local posx = position + ( direction * offset );
    ObjSet( "obj_int_button_collectibles_root", { pos_x = posx } );
    local pos_obj = ObjGet( "obj_int_button_collectibles_other" ).pos_x;
    ObjSet( "obj_int_button_collectibles_other", { pos_x = pos_obj - posx } );

  end;

end;
--******************************************************************************************
function private.EventAnimEnd ( event_id )

  if ( event_id == "show" ) then

    ObjSet( "obj_int_button_collectibles", { input = 1 } );
    
    ObjSet( "tmr_int_button_collectibles_hide", { endtrig = public.Hide, time = private.timer, playing = true } );

    ObjSet( "pfx_int_button_collectibles_anim", { active = true } );
    PartSysStart( "pfx_int_button_collectibles_anim" );

  elseif ( event_id == "hide" ) then

    ObjSet( "obj_int_button_collectibles_root", { active = true, visible = false } );

    ObjSet( "obj_int_button_collectibles", { input = false } );

    ObjSet( "pfx_int_button_collectibles_anim", { active = true } );

  end;
  
end;
--******************************************************************************************
function private.GoCollectibles ()

  cmn.EnterRmWithoutInterface( "rm_collectibles" );
  
  if ( common.GetCurrentSubRoom() ) then

     common.CloseSubRoom();

  end;

  ObjSet( "tmr_int_button_collectibles_hide", { endtrig = public.Hide, time = 0.01, playing = true } );

end;
--******************************************************************************************
function private.ShowAnim ( event_id, name )

  local obj       = "obj_int_button_collectibles";
  local func_end  = function () private.EventAnimEnd( event_id ); end;
  local tme       = private.MOVE_ANIM_TIME;
  local level     = common.GetObjectPrefix( name );

  ObjSet( "obj_int_button_collectibles_root",                 { visible = true } );
  ObjSet( "txt_int_button_collectibles_button_name", { text = "str_collectibles_name_"..name } );
  ObjSet( "spr_int_button_collectibles_icon",        { res = "assets/interface/resources/collect/level"..level.."/mask_"..name } );
  ObjStopAnimate( obj, "pos_x" ); 
  ObjSet( "tmr_int_button_collectibles_hide", { playing = false } );

  local current_posx = ObjGet( obj ).pos_x; 
  local distance = math.abs( private.MOVE_ANIM_POS_X_BEG - current_posx );

  common_impl.PlayAudio( "sfx", "aud_clk_morfing", 0, "", 0 );

  if ( distance ~= 0 ) then

    local overtime = distance * ( tme / ( private.MOVE_ANIM_POS_X_END - private.MOVE_ANIM_POS_X_BEG ) );
    ObjAnimate( obj, "pos_x", 0, 0, func_end,
    {
      0.0,          2, current_posx,
      overtime,     2, private.MOVE_ANIM_POS_X_BEG,
      overtime+tme, 2, private.MOVE_ANIM_POS_X_END
    } );
  
  else
  
    ObjAnimate( obj, "pos_x", 0, 0, func_end,
    {
      0.0, 2, private.MOVE_ANIM_POS_X_BEG,
      tme, 2, private.MOVE_ANIM_POS_X_END
    } );
  
  end;

end;
--******************************************************************************************
function private.HideAnim ( event_id )

  local obj           = "obj_int_button_collectibles";
  local func_end      = function () private.EventAnimEnd( event_id ); end;
  local tme           = private.MOVE_ANIM_TIME; 
  local current_posx  = private.MOVE_ANIM_POS_X_END; 

  ObjStopAnimate( obj, "pos_x" ); 
  current_posx = ObjGet( obj ).pos_x; 
  
  ObjAnimate( obj, "pos_x", 0, 0, func_end,
  {
    0.0, 1, ObjGet( obj ).pos_x,
    tme, 1, private.MOVE_ANIM_POS_X_BEG
  } );

  ObjSet( "tmr_int_button_collectibles_hide", { playing = false } );

end;
--******************************************************************************************
function private.MouseEnter ()

  SetCursor( CURSOR_HAND );

  local obj = "spr_int_button_collectibles_button_focus";
  local alp = ObjGet( obj ).alp;
  local tme = ( 1 - alp ) * private.MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 2, alp,
    tme, 2, 1
  } );

  ObjSet( "tmr_int_button_collectibles_hide", { active = false } );

end;
--******************************************************************************************
function private.MouseLeave ()

  SetCursor( CURSOR_DEFAULT );

  local obj = "spr_int_button_collectibles_button_focus";
  local alp = ObjGet( obj ).alp;
  local tme = alp * private.MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 1, alp,
    tme, 1, 0
  } );

  ObjSet( "tmr_int_button_collectibles_hide", { active = true } );

end;
--******************************************************************************************