-- name=mg_transformer
--*********************************************************************************************************************
function public.Init ()

  private.LogicInit();
  private.InitAnimations();

  private.room_objname = "mg_transformer";

  ObjMultiSet({ 
    { "spr_transformer_mechanismpart", { event_mdown = private.get_mechanismpart }},
    { "obj_transformer_click",         { event_mdown = private.LeverClick }},
  });

  --------------------------------------------------------------------------------------------
  -- function win_transformer () end;
  --------------------------------------------------------------------------------------------

  common_impl.hint[ "win_transformer" ] = {
    type = "win",
    room = "rm_powerunit",
    zz = "mg_transformer",
    zz_gate = "gmg_powerunit_transformer"
  };

  local win_transformer = function ()

    if ( cmn.IsEventDone( "win_transformer" ) ) then  

      ObjSet( "obj_transformer_buttons", { input = false } );
      ObjDelete( "obj_transformer_click" );

    end;

  end;

  local win_transformer_delete = function ()

    if ( cmn.IsEventDone( "win_transformer" ) ) then  

      ObjSet( "gzz_powerunit_transformer", { visible = true, input = true } );
      ObjDelete( "gmg_powerunit_transformer" );

    end;

  end;

  cmn.AddSubscriber( "win_transformer", win_transformer, private.room_objname );
  cmn.AddSubscriber( "win_transformer", win_transformer_delete, "rm_powerunit" );
  
  --------------------------------------------------------------------------------------------
end;

--*********************************************************************************************************************
--*********************************************************************************************************************
-- function *** ANIMATIONS *** () end;
--*********************************************************************************************************************
--*********************************************************************************************************************

function private.InitAnimations ()

  for i = 1, 5, 1 do 

    ObjAnimate( "spr_transformer_lamp_"..i, "alp",1,0, "", { 0, 3, 1,    0.2, 3, 0.9,    0.4, 3, 1 } );
    ObjAnimate( "spr_transformer_wire_"..i, "alp",1,0, "", { 0, 3, 1,    0.8, 3, 0.6,    1.6, 3, 1 } );

  end;

end;

--*********************************************************************************************************************
--*********************************************************************************************************************

function public.Open () 

  if not cmn.IsEventDone( "win_transformer" ) then

    cmn.MiniGameShow();
    common_impl.PassingMgStart( "transformer" );

  end;

end;
--*********************************************************************************************************************
function public.PreOpen ()

  private.Load();
  private.Update();

end;
--*********************************************************************************************************************
function public.Close ()
end;
--*********************************************************************************************************************
function public.PreClose ()

  if not cmn.IsEventDone( "win_transformer" ) then

    cmn.MiniGameHide();
    common_impl.PassingMgStop( "transformer" );

  end;

end;


--*********************************************************************************************************************
--*********************************************************************************************************************
-- function *** MINIGAME LOGIC *** () end;
--*********************************************************************************************************************
--*********************************************************************************************************************



private.state = {};

private.state_light = {};

private.state_win = { 0, 0, 0, 0, 0 };


-- information about every button: its rotate angle and according angle 
-- list of burning wires and burning lamps, linked with button 

private.links = { 
                  -- first button 
                  { 
                    degrees_0   = { wires = {},       lamp_burns = {} }, 
                    degrees_90  = { wires = { 1, 2 }, lamp_burns = { 1, 2 } }, 
                    degrees_180 = { wires = { 2 },    lamp_burns = { 2 } }, 
                    degrees_270 = { wires = { 1 },    lamp_burns = { 1 } } 
                  },

                  -- second button 
                  { 
                    degrees_0   = { wires = {},       lamp_burns = {} }, 
                    degrees_90  = { wires = { 2 },    lamp_burns = { 2 } }, 
                    degrees_180 = { wires = {},       lamp_burns = {} }, 
                    degrees_270 = { wires = { 2 },    lamp_burns = { 2 } } 
                  },

                  -- third button 
                  { 
                    degrees_0   = { wires = {},       lamp_burns = {} }, 
                    degrees_90  = { wires = { 3 },    lamp_burns = { 3 } }, 
                    degrees_180 = { wires = { 3 },    lamp_burns = { 3 } }, 
                    degrees_270 = { wires = {},       lamp_burns = {} } 
                  },

                  -- fourth button 
                  { 
                    degrees_0   = { wires = {},       lamp_burns = {} }, 
                    degrees_90  = { wires = { 4, 5 }, lamp_burns = { 4, 5 } }, 
                    degrees_180 = { wires = { 5 },    lamp_burns = { 5 } }, 
                    degrees_270 = { wires = { 4 },    lamp_burns = { 4 } } 
                  },

                  -- fifth button 
                  { 
                    degrees_0   = { wires = {},       lamp_burns = {} }, 
                    degrees_90  = { wires = { 5 },    lamp_burns = { 5 } }, 
                    degrees_180 = { wires = {},       lamp_burns = {} }, 
                    degrees_270 = { wires = { 5 },    lamp_burns = { 5 } } 
                  } 

                };


local TRANSFORMER_BUTTON_NAME = "spr_transformer_button_";
local TRANSFORMER_WIRE_NAME = "spr_transformer_wire_";
local TRANSFORMER_LAMP_NAME = "spr_transformer_lamp_";

local TRANSFORMER_BUTTON_COUNT = 5;
local TRANSFORMER_BUTTON_ANGLE_DEGREE = 90;
local TRANSFORMER_BUTTON_ANGLE_RADIAN = -1.57;

local isWin = true;

--*******************************************************************************************************************
function private.Default ()

  private.state = { 0, 1, 2, 1, 3 };

  private.state_light = { "true", "true", "true", "true", "true" };

end; 


--*******************************************************************************************************************
function private.Load ()

  private.state = ng_global.progress[ "std" ][ "win_transformer" ].state;

  private.state_light = ng_global.progress[ "std" ][ "win_transformer" ].state_light;

  if not private.state and not private.state_light then 
    private.Default();
  end; 

end;


--*******************************************************************************************************************
function private.Save ()

  ng_global.progress[ "std" ][ "win_transformer" ].state = private.state;

  ng_global.progress[ "std" ][ "win_transformer" ].state_light = private.state_light;

end;


--*******************************************************************************************************************
function private.LogicInit ()   

  for i = 1, TRANSFORMER_BUTTON_COUNT, 1 do

    local curButton = TRANSFORMER_BUTTON_NAME..i;
    ObjSet( curButton, { 
                          input = 1,
                          event_mdown = function () private.Click( i ); end,
                          event_menter = function () private.Enter(); end,
                          event_mleave = function () private.Leave(); end
                       } );

  end;

end;


--*******************************************************************************************************************
function private.Enter ()
 
  SetCursor( CURSOR_HAND );

end;


--*******************************************************************************************************************
function private.Leave ()  

  SetCursor( CURSOR_DEFAULT );

end;


--*******************************************************************************************************************
-- giving field name in data array by angle value for button   

function private.GiveFieldNameByAngle ( angle )

  local number = angle*TRANSFORMER_BUTTON_ANGLE_DEGREE;
  local field = "degrees_"..number;

  return field;

end;


--*******************************************************************************************************************
-- all wires and lamps are fading 

function private.ClearWiresAndLamps ()

  for i = 1, TRANSFORMER_BUTTON_COUNT, 1 do

    local curWire = TRANSFORMER_WIRE_NAME..i;
    ObjSet( curWire, { visible = false } );

    local curLamp = TRANSFORMER_LAMP_NAME..i;
    ObjSet( curLamp, { visible = false } );

  end;

end;


--*******************************************************************************************************************
function private.Click ( id )

  cmn.Lock( true );

  local func_end = function ()

    cmn.Lock( false );
    private.Update();

  end;

  local curButton = TRANSFORMER_BUTTON_NAME..id;
  local prevAngle = private.state[id]*TRANSFORMER_BUTTON_ANGLE_RADIAN;

  private.state[id] = private.state[id] + 1;
  local nextAngle = private.state[id]*TRANSFORMER_BUTTON_ANGLE_RADIAN;

  -- if angle == 2*PI, it will reset   
  if private.state[id] > 3 then
    private.state[id] = 0;
  end;

  private.Save();

  common_impl.PlayAudio( "sfx", "level3/sfx/aud_single_clk_tumbler", 0, "", 0 );

  ObjAnimate( curButton, "ang",0,0, func_end, { 0, 0, prevAngle,    0.6, 0, nextAngle } );

end;


--*******************************************************************************************************************
function private.Update ()

  isWin = true;
  private.ClearWiresAndLamps();

  for i = 1, TRANSFORMER_BUTTON_COUNT, 1 do

    local curButton = TRANSFORMER_BUTTON_NAME..i;
    local curButtonAngle = TRANSFORMER_BUTTON_ANGLE_RADIAN*private.state[i];

    ObjSet( curButton, { ang = curButtonAngle } );

-------------------------

    if ( i == 2 and ( private.state[i] == 1 or private.state[i] == 3 ) ) then 

      if private.state[1] == 0 or private.state[1] == 1 then

        ObjSet( TRANSFORMER_WIRE_NAME.."1", { visible = true } );
        ObjSet( TRANSFORMER_LAMP_NAME.."1", { visible = true } );

      end;

    end;

    if ( i == 5 and ( private.state[i] == 1 or private.state[i] == 3 ) ) then 

      if private.state[4] == 0 or private.state[4] == 1 then

        ObjSet( TRANSFORMER_WIRE_NAME.."4", { visible = true } );
        ObjSet( TRANSFORMER_LAMP_NAME.."4", { visible = true } );

      end;

    end;

-------------------------

    local degrees = private.GiveFieldNameByAngle( private.state[i] );
    local wires = private.links[i][degrees].wires;

    for j = 1, #wires, 1 do

      isWin = false;
      local curWire = TRANSFORMER_WIRE_NAME..wires[j];
      ObjSet( curWire, { visible = true } );

    end;

-------------------------

    local lamps = private.links[i][degrees].lamp_burns;

    for k = 1, #lamps, 1 do

      isWin = false;
      local curLamp = TRANSFORMER_LAMP_NAME..lamps[k];
      ObjSet( curLamp, { visible = true } );

    end;

  end;

----------------------------------------------------------------------------------------
  for q=1,5 do
    if ( ObjGet(TRANSFORMER_LAMP_NAME..q).visible == false ) then
      if ( private.state_light[q] == "false"  ) then
        common_impl.PlayAudio( "sfx", "level3/sfx/aud_single_clk_button2", 0, "", 0 );
        private.state_light[q] = "true";
      end;
    else
      if ( private.state_light[q] == "true"  ) then
        common_impl.PlayAudio( "sfx", "level3/sfx/aud_single_clk_lamp", 0, "", 0 );
        private.state_light[q] = "false";
      end;
    end;
  end;
----------------------------------------------------------------------------------------

end;


--*******************************************************************************************************************
function private.CheckWin ()

  return isWin;  
    
end;


--*********************************************************************************************************************
function private.Win ()  

  local func_end = function ()

    cmn.SetEventDone( "win_transformer" );
    cmn.CallEventHandler( "win_transformer" );
    common_impl.PassingMgWin( "transformer" );

    cmn.Lock( false );
    cmn.GotoRoom( "rm_powerunit" );

  end;

  cmn.MiniGameHide();
  cmn.Lock( true );

  for i = 1, TRANSFORMER_BUTTON_COUNT, 1 do 

    local curLamp = TRANSFORMER_LAMP_NAME..i;
    ObjStopAnimate( curLamp, 8 );

    ObjSet( curLamp, { alp = 1, visible = true } );

    local anim_end = "";
    if i == TRANSFORMER_BUTTON_COUNT then 

      anim_end = func_end;      
      rm_powerunit.open_transformer = true;

    end;

    ObjAnimate( curLamp, "alp",0,0, anim_end, { 0, 3, 1,    0.4, 3, 0,    0.8, 3, 1,    1.6, 3, 0 } );

  end;

end;


--*********************************************************************************************************************
function private.LeverClick ()

  if private.CheckWin() then 

    local func_end = function ()
      private.Win(); 
    end;

    common_impl.PlayAudio( "sfx", "level3/sfx/aud_win_mg_transformer", 0, "", 0 );

    ObjSet( "obj_transformer_click", { input = false } );
    ObjAnimate( "spr_transformer_lever", "pos_y",0,0, func_end, { 0, 2, 220,    0.6, 2, 480 } );

  else 

    local func_end = function ()

      cmn.Lock( false );
      ObjSet( "spr_transformer_lever", { alp = 1 } );
      ObjSet( "anm_mg_transformer_idle_speed", { alp = 0 } );

    end;

    local func_end_2 = function ()
      ObjSet( "spr_transformer_lever", { alp = 0 } );
    end;

    cmn.Lock( true );

    common_impl.PlayAudio( "sfx", "level3/sfx/aud_clk_lever_mg", 0, "", 0 );

    ObjAnimate( "anm_mg_transformer_idle_speed", "alp",0,0, func_end_2, { 0, 0, 0,    0.1, 0, 1 } );
    AnimPlay( "anm_mg_transformer_idle_speed", "DoAnim", func_end );

  end;

end;


--*********************************************************************************************************************
function public.Reset ()

  cmn.Lock( true );

  local oldAngle = {};

  for i = 1, TRANSFORMER_BUTTON_COUNT, 1 do
    oldAngle[i] = private.state[i];
  end; 

  private.Default();
  private.Save();

  for j = 1, TRANSFORMER_BUTTON_COUNT, 1 do

    local prevAngle = TRANSFORMER_BUTTON_ANGLE_RADIAN*oldAngle[j];
    local nextAngle = TRANSFORMER_BUTTON_ANGLE_RADIAN*private.state[j];
    local curButon = TRANSFORMER_BUTTON_NAME..j;

    local func_end = "";

    if j == TRANSFORMER_BUTTON_COUNT then 

      func_end = function ()

        cmn.Lock( false );
        private.Update();

      end;

    end;  

    if prevAngle > nextAngle then 

      prevAngle = prevAngle - 2*math.pi;

    end; 

    ObjAnimate( curButon, "ang",0,0, func_end, { 0, 2, prevAngle,    0.4, 2, nextAngle } );

  end; 

end;


--*********************************************************************************************************************
function public.Skip ()

port.skip_mg = true;
  cmn.Lock( true );

  local oldAngle = {};

  for i = 1, TRANSFORMER_BUTTON_COUNT, 1 do
    oldAngle[i] = private.state[i];
  end; 

  private.state = private.state_win;

  for j = 1, TRANSFORMER_BUTTON_COUNT, 1 do

    local prevAngle = TRANSFORMER_BUTTON_ANGLE_RADIAN*oldAngle[j];
    local nextAngle = TRANSFORMER_BUTTON_ANGLE_RADIAN*private.state[j];
    local curButon = TRANSFORMER_BUTTON_NAME..j;

    local func_end = "";

    if j == TRANSFORMER_BUTTON_COUNT then 

      func_end = function ()

        cmn.Lock( false );
        private.Update();
        private.LeverClick();

      end;

    end;  

    if prevAngle > nextAngle then 

      prevAngle = prevAngle - 2*math.pi;

    end; 

    ObjAnimate( curButon, "ang",0,0, func_end, { 0, 2, prevAngle,    0.4, 2, nextAngle } );

  end; 

end;

--*********************************************************************************************************************










