-- name=mg_hoteldoor
--*********************************************************************************************************************
function public.Init ()

  private.room_objname = "mg_hoteldoor";

  -- balls creating for all five levels 
  private.LogicInit( 1 ); 
  private.LogicInit( 2 ); 
  private.LogicInit( 3 ); 

  --------------------------------------------------------------------------------------------

  for i = 1, 3, 1 do

    local curLamp = "spr_hoteldoor_lamp_light_"..i;
    ObjAnimate( curLamp, "alp",1,0, "", { 0, 3, 1,    0.9, 3, 0.4,    2.1, 3, 0.8,    3.5, 3, 0.5,    5.3, 3, 1 } );

  end; 

  --------------------------------------------------------------------------------------------
  -- function win_hoteldoor () end;
  --------------------------------------------------------------------------------------------
  common_impl.hint[ "win_hoteldoor" ] = {
    type = "win",
    room = "rm_porch_ext",
    zz = private.room_objname,
    zz_gate = "gmg_porch_ext_hoteldoor",
  };
  local win_hoteldoor = function ()

    if ( cmn.IsEventDone( "win_hoteldoor" ) ) then  

      ObjSet( "obj_hoteldoor_level_general", { alp = 0, visible = 0, input = false } );
      ObjSet( "spr_hoteldoor_lever_handle", { input = false } ); 

    end;

  end;

  local win_hoteldoor_delete = function ()

    if ( cmn.IsEventDone( "win_hoteldoor" ) ) then  

      ObjDelete( "spr_porch_ext_frontdoor_door" );
      ObjDelete( "gmg_porch_ext_hoteldoor" );
      ObjSet( "grm_porch_ext_hall_ext", { input = true, visible = true, active = true, alp = 1 } );
      if port.isPortable() == 1 then
          port.GrmEventLightOn("grm_porch_ext_hall_ext_focus");
      end;
    end;

  end;

  cmn.AddSubscriber( "win_hoteldoor", win_hoteldoor, private.room_objname );
  cmn.AddSubscriber( "win_hoteldoor", win_hoteldoor_delete, "rm_porch_ext" );
  
  --------------------------------------------------------------------------------------------
end;


--*********************************************************************************************************************
function public.Open () 

  int_slider.InitSlider( private.room_objname );

  if not cmn.IsEventDone( "win_hoteldoor" ) then

    cmn.MiniGameShow();
    common_impl.PassingMgStart();

  end;

  cmn.Lock( true );
  ObjSet( "obj_hoteldoor_level_general", { input = false } );

  -- checking on necessity of balls binding through holes  
  private.Bonding( 1 ); 

end;


--*********************************************************************************************************************
function public.PreOpen ()

  if ( not ng_global.progress[ "ext" ][ "win_hoteldoor" ].level ) then 
    ng_global.progress[ "ext" ][ "win_hoteldoor" ].level = 1;
  end;

  private.Load();
  private.LevelChoosing(); 
  private.Update();

end;


--*********************************************************************************************************************
function public.Close ()
end;


--*********************************************************************************************************************
function public.PreClose ()

  if not cmn.IsEventDone( "win_hoteldoor" ) then

    cmn.MiniGameHide();
    common_impl.PassingMgStop();

  end;

end;


--*********************************************************************************************************************
--*********************************************************************************************************************
-- function *** MINIGAME LOGIC *** () end;
--*********************************************************************************************************************
--*********************************************************************************************************************

local HOTELDOOR_LEVEL_NAME = "obj_hoteldoor_level_";
local HOTELDOOR_BALL_NAME = "spr_hoteldoor_ball_";

local HOTELDOOR_DIGIT_NAME = "spr_hoteldoor_didits_light_";
local HOTELDOOR_LAMP_NAME = "spr_hoteldoor_lamp_light_";

local HOTELDOOR_LEVEL_COUNT = 3;
local HOTELDOOR_BALL_COUNT_LEVEL_1 = 79;
local HOTELDOOR_BALL_COUNT_LEVEL_2 = 61;
local HOTELDOOR_BALL_COUNT_LEVEL_3 = 142;

local HOTELDOOR_FIELD_HORIZONTAL = 19;
local HOTELDOOR_FIELD_VERTICAL = 19;

local HOTELDOOR_START_X = 192;
local HOTELDOOR_START_Y = -2;
local HOTELDOOR_WIDTH = 32;

local HOTELDOOR_DEADLOCK = -1;
local HOTELDOOR_EMPTY = 0;
local HOTELDOOR_RED_BALL = 1;
local HOTELDOOR_ORANGE_BALL = 2;
local HOTELDOOR_GREEN_BALL= 3;
local HOTELDOOR_BLUE_BALL = 4;
local HOTELDOOR_PURPLE_BALL = 5;

local HOTELDOOR_DELETING_BALL = 1;


local curBallCount = 0;
local curLevel = 0;
local levelWin = true;


-- RGB array for balls colors 
private.colors_rgb = { 
                        red = { r = 1, g = 0, b = 0 }, 
                        orange = { r = 1, g = 0.5, b = 0 }, 
                        green = { r = 0, g = 1, b = 0 }, 
                        indigo = { r = 0, g = 0, b = 1 },  
                        purple = { r = 1, g = 0, b = 1 },
                        blue = { r = 0, g = 1, b = 1 },  
                        yellow = { r = 1, g = 1, b = 0 }  
                     };

-- array for marking balls cells, if 1 -- ball in current cell must be deleted 
private.marked = { 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }, 
                     { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }                          
                  };



--*******************************************************************************************************************

-- array of balls positions for the first level 
private.field_level_1 = {
                           { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  2,  3,  4,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 17,  0,  0, -1 }, 
                           { -1,  0,  0, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30,  0,  0, -1 }, 
                           { -1,  0, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45,  0, -1 }, 
                           { -1, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, -1 }, 
                           { -1, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, -1 }, 
                           { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 }       
                       };


-- array of colors for corresponding balls for the first level, where: 
-- (-1) - deadlock 
-- 0 - empty
-- 1 - red 
-- 2 - orange 
-- 3 - green 
-- 4 - indigo 
-- 5 - purple 
-- 6 - blue
-- 7 - yellow 
private.field_colors_level_1 = { 
                                 2, 2, 2, 2, 1, 1, 1, 1, 2, 3, 3, 3, 2, 1, 1, 1, 1,
                                 1, 5, 1, 1, 3, 3, 3, 3, 3, 1, 1, 5, 1, 1, 5, 5, 5,
                                 1, 1, 1, 3, 1, 1, 1, 5, 5, 5, 1, 1, 5, 5, 5, 5, 5, 
                                 1, 1, 1, 1, 1, 5, 5, 5, 5, 5, 1, 1, 4, 4, 4, 4, 4, 
                                 1, 3, 3, 3, 1, 4, 4, 4, 4, 4, 1 
                               };


--*******************************************************************************************************************

-- array of balls positions for the second level 
private.field_level_2 = { 
                           { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  3,  4,  5,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  6,  7,  8,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  9, 10, 11, 12, 13,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0, 14, 15, 16, 17, 18,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0, 19, 20, 21, 22, 23, 24, 25,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0, 26, 27, 28, 29, 30, 31, 32,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0, 33, 34, 35, 36, 37, 38, 39, 40, 41,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0, 42, 43, 44, 45, 46, 47, 48, 49, 50,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61,  0,  0,  0, -1 }, 
                           { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 }                          
                        };

-- array of colors for corresponding balls for the second level, where: 
-- (-1) - deadlock 
-- 0 - empty
-- 1 - red 
-- 2 - orange 
-- 3 - green 
-- 4 - indigo 
-- 5 - purple 
-- 6 - blue
-- 7 - yellow 
private.field_colors_level_2 = { 
                                 4, 6, 4, 5, 4, 6, 3, 6, 4, 5, 1, 5, 4, 6, 3, 1, 3, 
                                 6, 4, 5, 1, 1, 1, 5, 4, 6, 3, 3, 3, 3, 3, 6, 4, 5, 
                                 5, 5, 5, 5, 5, 5, 4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4, 
                                 4, 4, 4, 4, 4, 4, 4, 4, 4, 4 
                               };


--*******************************************************************************************************************

-- array of balls positions for the third level 
private.field_level_3 = { 
                           { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  0,  0,  2,  3,  4,  0,  0,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0,  0,  5,  6,  7,  8,  9, 10, 11,  0,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0,  0, 12, 13, 14, 15, 16, 17, 18, 19, 20,  0,  0,  0,  0, -1 }, 
                           { -1,  0,  0,  0, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,  0,  0,  0, -1 }, 
                           { -1,  0,  0, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44,  0,  0, -1 }, 
                           { -1,  0, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58,  0,  0, -1 }, 
                           { -1, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74,  0, -1 }, 
                           { -1, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, -1 }, 
                           { -1, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103,104,105,106,107,108, -1 }, 
                           { -1,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125, -1 }, 
                           { -1,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142, -1 }, 
                           { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 }                          
                        };

-- array of colors for corresponding balls for the third level, where:  
-- (-1) - deadlock 
-- 0 - empty
-- 1 - red 
-- 2 - orange 
-- 3 - green 
-- 4 - indigo 
-- 5 - purple 
-- 6 - blue
-- 7 - yellow  
private.field_colors_level_3 = { 
                                 7, 2, 2, 2, 2, 2, 7, 1, 7, 2, 2, 2, 4, 4, 4, 4, 4,
                                 4, 4, 2, 2, 6, 2, 6, 6, 2, 6, 6, 2, 6, 2, 6, 6, 6, 
                                 2, 6, 6, 2, 6, 6, 2, 6, 6, 6, 6, 6, 6, 6, 2, 6, 6, 
                                 2, 6, 6, 2, 6, 6, 6, 6, 6, 6, 6, 6, 2, 6, 6, 2, 6, 
                                 6, 2, 6, 6, 6, 6, 6, 6, 6, 6, 6, 2, 6, 6, 2, 6, 6, 
                                 2, 6, 6, 6, 6, 6, 6, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 
                                 1, 1, 6, 6, 6, 6, 6, 6, 4, 4, 4, 4, 2, 2, 2, 2, 2, 
                                 4, 4, 4, 4, 6, 6, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 
                                 5, 5, 5, 5, 5, 5 
                                };

--*******************************************************************************************************************
-- getting default state for current level 
function private.Default () 

  local level = "field_level_"..ng_global.progress[ "ext" ][ "win_hoteldoor" ].level;
  private.field = {};
  private.GettingArrayCopy( private[ level ], private.field )  

  private.removed_balls = {};

end; 


--*******************************************************************************************************************
-- current level loading 
function private.Load ()

  private.field = ng_global.progress[ "ext" ][ "win_hoteldoor" ].field;
  private.removed_balls = ng_global.progress[ "ext" ][ "win_hoteldoor" ].removed_balls; 

  curLevel = ng_global.progress[ "ext" ][ "win_hoteldoor" ].level;
  ballCount = private.GettingBallCountByLevel( curLevel );

  local level = "field_colors_level_"..curLevel;
  private.field_colors = private[ level ]; 

  if ( not private.field or not private.removed_balls ) then 
    private.Default();
  end; 

end;


--*******************************************************************************************************************
function private.Save ()

  ng_global.progress[ "ext" ][ "win_hoteldoor" ].field = private.field;
  ng_global.progress[ "ext" ][ "win_hoteldoor" ].removed_balls = private.removed_balls;

end;


--*******************************************************************************************************************
-- showing of current level balls 
function private.LevelChoosing ()

  for i = 1, HOTELDOOR_LEVEL_COUNT, 1 do 

    local levelName = HOTELDOOR_LEVEL_NAME..i;
    ObjSet( levelName, { active = false, alp = 0, visible = false, input = false } );

    if i == curLevel then 
      ObjSet( levelName, { active = true, alp = 1, visible = true, input = true } ); 
    end; 

  end;

end;


--*******************************************************************************************************************
-- getting values of array-source 
function private.GettingArrayCopy ( sourceArray, copyArray )  

  for i = 1, #sourceArray, 1 do

    table.insert( copyArray, {} );

    for j = 1, #sourceArray[ i ], 1 do 

      table.insert( copyArray[ i ], sourceArray[ i ][ j ] );

    end;

  end; 

end; 


--*******************************************************************************************************************
-- color definition 
function private.GettingColorById ( id )  

  local result = "red";

  if id == 1 then 
    result = "red"; 

  elseif id == 2 then 
    result = "orange"; 

  elseif id == 3 then 
    result = "green"; 

  elseif id == 4 then 
    result = "indigo"; 

  elseif id == 5 then 
    result = "purple"; 

  elseif id == 6 then 
    result = "blue"; 

  elseif id == 7 then 
    result = "yellow"; 

  end;

  return result;

end;


--*******************************************************************************************************************
-- balls count definition by current level 
function private.GettingBallCountByLevel ( level )  

  local result = HOTELDOOR_BALL_COUNT_LEVEL_1;

  if level == 1 then 
    result = HOTELDOOR_BALL_COUNT_LEVEL_1; 

  elseif level == 2 then 
    result = HOTELDOOR_BALL_COUNT_LEVEL_2; 

  elseif level == 3 then 
    result = HOTELDOOR_BALL_COUNT_LEVEL_3; 

  end;

  return result;

end;


--*******************************************************************************************************************
-- balls creating for every level 
function private.LogicInit ( level ) 

  local ballCountInit = private.GettingBallCountByLevel( level );

  local levelColor = "field_colors_level_"..level;
  private.field_colors = private[ levelColor ]; 

  for i = 1, ballCountInit, 1 do

    local curBallUp = HOTELDOOR_BALL_NAME..level.."_"..i.."_up";

    ObjCreate( curBallUp, "spr" );
    ObjAttach( curBallUp, "obj_hoteldoor_level_"..level );
    ObjSet( curBallUp, { 
                          res = "assets/levels/levelext/mg_hoteldoor/layers/counter_up_side" 
                       } );
--------------------------------------
--------------------------------------
    local curBall = HOTELDOOR_BALL_NAME..level.."_"..i;
    ObjCreate( curBall, "spr" );
    ObjAttach( curBall, curBallUp );

    local curColor = private.GettingColorById( private.field_colors[i] );

    ObjSet( curBall, { 
                        input = true,
                        color_r = private.colors_rgb[ curColor ].r,
                        color_g = private.colors_rgb[ curColor ].g,
                        color_b = private.colors_rgb[ curColor ].b, 
                        res = "assets/levels/levelext/mg_hoteldoor/layers/counter", 
                        event_mdown = function () private.Click( i ); end,
                        event_menter = function () private.Enter(); end,
                        event_mleave = function () private.Leave(); end   
                     } );

  end;

end;


--*******************************************************************************************************************
function private.Enter ()
 
  SetCursor( CURSOR_HAND );

end;


--*******************************************************************************************************************
function private.Leave ()  

  SetCursor( CURSOR_DEFAULT );

end;


--*******************************************************************************************************************
function private.Click ( id ) 

  cmn.Lock( true );
  ObjSet( "obj_hoteldoor_level_general", { input = false } );

  common_impl.PlayAudio( "sfx", "levelext/sfx/aud_mghoteldoor_clk1", 0, "", 0 );

  -- getting of ball position and deleting marked balls 
  local line, column = private.GettingLineAndColumnById( id );
  private.MarkingIdenticalColorBalls( line, column );

  for i = 1, HOTELDOOR_FIELD_VERTICAL, 1 do

    for j = 1, HOTELDOOR_FIELD_HORIZONTAL, 1 do

      if private.marked[ i ][ j ] == HOTELDOOR_DELETING_BALL then

        local curBallUp = HOTELDOOR_BALL_NAME..curLevel.."_"..private.field[ i ][ j ].."_up";         
        ObjAnimate( curBallUp, "scale_xy",0,0, "", { 0, 2, 1, 1,    0.2, 2, 0, 0 } );
 
        table.insert( private.removed_balls, private.field[ i ][ j ] ); 
        private.field[ i ][ j ] = 0; 
        private.Save(); 

      end; 

    end; 

  end; 

  -- falling of top balls and checking on necessity 
  -- of balls binding through holes  

  for k = 1, HOTELDOOR_FIELD_HORIZONTAL, 1 do

    private.MovingBallsDown( k );

    if k == HOTELDOOR_FIELD_HORIZONTAL then 

      local func_end = function ()

        private.Bonding( 1 );

      end;

      ObjAnimate( "obj_hoteldoor_gates", "scale_x",0,0, func_end, { 0, 0, 1,    0.4, 0, 1 } );

    end;

  end;

end;
--*******************************************************************************************************************
-- getting ball position by its ID 
function private.GettingLineAndColumnById ( id )

  local line = 2;
  local column = 2; 

  for i = 1, HOTELDOOR_FIELD_VERTICAL, 1 do

    for j = 1, HOTELDOOR_FIELD_HORIZONTAL, 1 do 

      if private.field[ i ][ j ] == id then 

        line = i;
        column = j; 
        break; 

      end; 

    end;

  end; 

  return line, column; 

end; 
--*******************************************************************************************************************
-- moving top balls, when corresponding bottom balls are deleting 
function private.MovingBallsDown ( columnId ) 

  local minArray = {};
  local maxArray = {};

  local minLine = 0;
  local maxLine = 0;

  for i = 1, HOTELDOOR_FIELD_VERTICAL - 1, 1 do

    if private.marked[ i ][ columnId ] == HOTELDOOR_DELETING_BALL then 

      if minLine == 0 then 

        minLine = i;
        table.insert( minArray, minLine );

      end; 

      if private.marked[ i + 1 ][ columnId ] ~= HOTELDOOR_DELETING_BALL then 

        maxLine = i;
        table.insert( maxArray, maxLine );
        minLine = 0;

      end;

    end; 

  end; 
----------------------------------
----------------------------------
  local movingDelta = 0; 

  if #minArray ~= 0 and #maxArray ~= 0 then

    for k = #minArray, 1, -1 do 

      local prevMax = 1;
      movingDelta = movingDelta + math.abs( minArray[ k ] - maxArray[ k ] ) + 1;

      if maxArray[ k - 1 ] then 
        prevMax = maxArray[ k - 1 ] + 1;
      end;

      for i = minArray[ k ] - 1, prevMax, -1 do 

        if private.field[ i ][ columnId ] ~= HOTELDOOR_DEADLOCK and 
        private.field[ i ][ columnId ] ~= HOTELDOOR_EMPTY then 

          local oldY = HOTELDOOR_START_Y + i*HOTELDOOR_WIDTH;
          local newY = HOTELDOOR_START_Y + ( i + movingDelta )*HOTELDOOR_WIDTH;

          local curBallUp = HOTELDOOR_BALL_NAME..curLevel.."_"..private.field[ i ][ columnId ].."_up";
          ObjAnimate( curBallUp, "pos_y",0,0, "", { 0, 3, oldY,    0.3, 3, newY } );

          private.field[ i + movingDelta ][ columnId ] = private.field[ i ][ columnId ];
          private.field[ i ][ columnId ] = 0;  
          private.Save(); 

        end; 

      end;

    end; 

  end;

end; 


--*******************************************************************************************************************
-- marking balls, if they have the same color, that current ball has 
function private.MarkingIdenticalColorBalls ( line, column ) 

  private.marked[ line ][ column ] = 1;
  local curColor = private.field_colors[ private.field[ line ][ column ] ];

  if private.field_colors[ private.field[ line ][ column - 1 ] ] == curColor then 

    local oldvalue = private.marked[ line ][ column - 1 ];
    private.marked[ line ][ column - 1 ] = 1;

    if oldvalue ~= HOTELDOOR_DELETING_BALL then 
      private.MarkingIdenticalColorBalls( line, column - 1 ); 
    end;

  end;
----------------------------------
----------------------------------
  if private.field_colors[ private.field[ line ][ column + 1 ] ] == curColor then 

    local oldvalue = private.marked[ line ][ column + 1 ];
    private.marked[ line ][ column + 1 ] = 1;

    if oldvalue ~= HOTELDOOR_DELETING_BALL then 
      private.MarkingIdenticalColorBalls( line, column + 1 ); 
    end;

  end;
----------------------------------
----------------------------------
  if private.field_colors[ private.field[ line - 1 ][ column ] ] == curColor then 

    local oldvalue = private.marked[ line - 1 ][ column ];
    private.marked[ line - 1 ][ column ] = 1;

    if oldvalue ~= HOTELDOOR_DELETING_BALL then 
      private.MarkingIdenticalColorBalls( line - 1, column ); 
    end;

  end;
----------------------------------
----------------------------------
  if private.field_colors[ private.field[ line + 1 ][ column ] ] == curColor then
 
    local oldvalue = private.marked[ line + 1 ][ column ];
    private.marked[ line + 1 ][ column ] = 1;

    if oldvalue ~= HOTELDOOR_DELETING_BALL then 
      private.MarkingIdenticalColorBalls( line + 1, column ); 
    end;

  end;

end;


--*******************************************************************************************************************
-- definition input value of current ball; input = true,
-- if nearest balls are exist and its count > 2 
function private.GettingInput ( line, column ) 

  local result = true; 
  local curColor = private.field_colors[ private.field[ line ][ column ] ];

  if ( private.field_colors[ private.field[ line - 1 ][ column ] ] ~= curColor and 
  private.field_colors[ private.field[ line + 1 ][ column ] ] ~= curColor and
  private.field_colors[ private.field[ line ][ column - 1 ] ] ~= curColor and 
  private.field_colors[ private.field[ line ][ column + 1 ] ] ~= curColor ) then 

    result = false;

  end;
----------------------------------
----------------------------------
  if ( column < HOTELDOOR_FIELD_HORIZONTAL - 1 ) then 

    if ( private.field_colors[ private.field[ line ][ column - 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line ][ column + 2 ] ] ~= curColor and 
    private.field_colors[ private.field[ line - 1 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line + 1 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line - 1 ][ column + 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line + 1 ][ column + 1 ] ] ~= curColor ) then

      result = false;

    end;

  end; 
----------------------------------
----------------------------------
  if ( column > 1 ) then 

    if ( private.field_colors[ private.field[ line ][ column + 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line ][ column - 2 ] ] ~= curColor and 
    private.field_colors[ private.field[ line - 1 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line + 1 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line - 1 ][ column - 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line + 1 ][ column - 1 ] ] ~= curColor ) then

      result = false;

    end;

  end;
----------------------------------
----------------------------------
  if ( line < HOTELDOOR_FIELD_VERTICAL - 1 ) then 

    if ( private.field_colors[ private.field[ line - 1 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line + 2 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line ][ column - 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line ][ column + 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line + 1 ][ column - 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line + 1 ][ column + 1 ] ] ~= curColor ) then

      result = false;

    end;

  end; 
----------------------------------
----------------------------------
  if ( line > 1 ) then

    if ( private.field_colors[ private.field[ line + 1 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line - 2 ][ column ] ] ~= curColor and 
    private.field_colors[ private.field[ line ][ column - 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line ][ column + 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line - 1 ][ column - 1 ] ] ~= curColor and 
    private.field_colors[ private.field[ line - 1 ][ column + 1 ] ] ~= curColor ) then

      result = false;

    end;

  end;

  return result; 

end;


--*******************************************************************************************************************
-- balls columns are bind, if holes are exist; 
-- checking starts from current column 
function private.Bonding ( columnId ) 

  local lastLine = HOTELDOOR_FIELD_VERTICAL - 1; 
  local minColumn = 0;
  local maxColumn = 0;

  for j = columnId, HOTELDOOR_FIELD_HORIZONTAL - 1, 1 do

    if private.field[ lastLine ][ j ] ~= HOTELDOOR_DEADLOCK then 

      if private.field[ lastLine ][ j ] == HOTELDOOR_EMPTY then  

        if minColumn == 0 then 
          minColumn = j; 
        end;

        if private.field[ lastLine ][ j + 1 ] ~= HOTELDOOR_EMPTY then 

          maxColumn = j;
          break; 

        end; 

      end; 

    end; 

  end;
----------------------------------
----------------------------------
  local movingDelta = math.abs( minColumn - maxColumn ) + 1; 

  if ( minColumn > 2 and maxColumn < HOTELDOOR_FIELD_HORIZONTAL - 1 ) then 

    for i = 1, HOTELDOOR_FIELD_VERTICAL, 1 do 

      for j = minColumn - 1, 1, -1 do 

        if private.field[ i ][ j ] ~= HOTELDOOR_DEADLOCK and 
        private.field[ i ][ j ] ~= HOTELDOOR_EMPTY then  

          local oldX = HOTELDOOR_START_X + j*HOTELDOOR_WIDTH;
          local newX = HOTELDOOR_START_X + ( j + movingDelta )*HOTELDOOR_WIDTH;

          local curBallUp = HOTELDOOR_BALL_NAME..curLevel.."_"..private.field[ i ][ j ].."_up";
          ObjAnimate( curBallUp, "pos_x",0,0, "", { 0, 3, oldX,    0.3, 3, newX } );
          
          private.field[ i ][ j + movingDelta ] = private.field[ i ][ j ];
          private.field[ i ][ j ] = 0;  
          private.Save(); 

        end; 

      end; 

      if i == HOTELDOOR_FIELD_VERTICAL then 

        local func_end = function ()

          private.Bonding( maxColumn + 1 );

        end;

        ObjAnimate( "obj_hoteldoor_gates", "alp",0,0, func_end, { 0, 0, 1,    0.4, 0, 1 } );

      end;

    end; 

  elseif minColumn == 2 then 

    private.Bonding( maxColumn + 1 ); 

  else 

    cmn.Lock( false ); 
    ObjSet( "obj_hoteldoor_level_general", { input = true } );
    private.Update();

  end; 

end; 


--*******************************************************************************************************************
-- array of marked balls reseting 
function private.ClearingMarkedArrays ()

  for i = 1, HOTELDOOR_FIELD_VERTICAL, 1 do

    for j = 1, HOTELDOOR_FIELD_HORIZONTAL, 1 do

      private.marked[ i ][ j ] = 0;

    end;

  end; 

end;


--*******************************************************************************************************************
-- next level starting 
function private.LeverClick () 

  local func_end = function ()

    cmn.Lock( false );

  end;

  ng_global.progress[ "ext" ][ "win_hoteldoor" ].level = curLevel + 1;
  ng_global.progress[ "ext" ][ "win_hoteldoor" ].field = nil;
  ng_global.progress[ "ext" ][ "win_hoteldoor" ].removed_balls = nil;  
  private.Load();
  private.LevelChoosing(); 
  private.Update();

  cmn.Lock( true ); 
  ObjAnimate( "spr_hoteldoor_lever_handle", "pos_y",0,0, "", { 0, 1, 226,    0.6, 1, 500,    1.2, 2, 226 } );
  ObjAnimate( "spr_hoteldoor_grid", "pos_y",0,0, func_end, { 0, 3, 381,    0.6, 3, 381,    1.8, 3, 900 } );

  common_impl.PlayAudio( "sfx", "level3/sfx/aud_single_clk_lamp", 0, "", 0 );
  common_impl.PlayAudio( "sfx", "levelext/sfx/aud_mghoteldoor_clk3", 0, "", 0 );

end; 


--*******************************************************************************************************************
function private.Update () 

  ObjSet( "spr_hoteldoor_lever_handle", { event_mdown = function() common_impl.ShowBbt( "bbt_mg_hoteldoor_clk_handle" ); end } );

  for i = 1, curLevel, 1 do 

    local curLamp = HOTELDOOR_LAMP_NAME..i;
    ObjSet( curLamp, { visible = true } );

  end;

  for j = 1, curLevel - 1, 1 do 

    local curDigit = HOTELDOOR_DIGIT_NAME..j;
    ObjSet( curDigit, { alp = 0 } );

  end;

  local func_end = function ()

    cmn.Lock( false );

  end;

  private.ClearingMarkedArrays();

  local levelWin = true;
  local isNeedAutoReset = true; 

  for i = 1, HOTELDOOR_FIELD_VERTICAL, 1 do 

    for j = 1, HOTELDOOR_FIELD_HORIZONTAL, 1 do

      if ( private.field[ i ][ j ] ~= HOTELDOOR_DEADLOCK and 
      private.field[ i ][ j ] ~= HOTELDOOR_EMPTY ) then 

        levelWin = false; 

        local curBallUp = HOTELDOOR_BALL_NAME..curLevel.."_"..private.field[ i ][ j ].."_up";
        local curX = HOTELDOOR_START_X + j*HOTELDOOR_WIDTH;
        local curY = HOTELDOOR_START_Y + i*HOTELDOOR_WIDTH;
        local curInput = private.GettingInput( i, j );
        isNeedAutoReset = isNeedAutoReset and ( not curInput );

        ObjSet( curBallUp, { alp = 1, visible = true, scale_x = 1, scale_y = 1, 
                             pos_x = curX, pos_y = curY, input = curInput } );  

      end;     

    end;

  end;

  for k = 1, #private.removed_balls, 1 do

    local curBallUp = HOTELDOOR_BALL_NAME..curLevel.."_"..private.removed_balls[ k ].."_up";
    ObjSet( curBallUp, { alp = 0, visible = false, input = false } );

  end;
----------------------------------
----------------------------------
  -- local victory of level 
  if levelWin then 
  
    local curDigit = HOTELDOOR_DIGIT_NAME..ng_global.progress[ "ext" ][ "win_hoteldoor" ].level;
    ObjAnimate( curDigit, "alp",0,0, "", { 0, 0, 1,    0.4, 0, 0 } );

    if curLevel < HOTELDOOR_LEVEL_COUNT then 

      cmn.Lock( true );
      ObjSet( "spr_hoteldoor_lever_handle", { event_mdown = function() private.LeverClick(); end } );
      ObjAnimate( "spr_hoteldoor_grid", "pos_y",0,0, func_end, { 0, 3, 900,    0.8, 3, 381 } );

      common_impl.PlayAudio( "sfx", "levelext/sfx/aud_mghoteldoor_clk2", 0, "", 0 );

    else 

      private.Win();

    end; 

  -- when level game is over 
  elseif isNeedAutoReset then 

    ng_global.progress[ "ext" ][ "win_hoteldoor" ].field = nil;
    ng_global.progress[ "ext" ][ "win_hoteldoor" ].removed_balls = nil;  
    private.Load();
    private.LevelChoosing(); 
    private.Update();

    cmn.Lock( true ); 
    ObjAnimate( "spr_hoteldoor_grid", "pos_y",0,0, func_end, { 0, 3, 900,    0.8, 3, 381,    1.4, 3, 381,    2.6, 3, 900 } );

    common_impl.PlayAudio( "sfx", "levelext/sfx/aud_mghoteldoor_clk2", 0, "", 0 );

    common_impl.PlayAudio( "sfx", "level3/sfx/aud_single_clk_lamp", 0, "", 0 );

  end; 

end;


--*********************************************************************************************************************
function private.Win ()

  local func_end = function ()

    cmn.Lock( false );
    cmn.SetEventDone( "win_hoteldoor" );
    cmn.CallEventHandler( "win_hoteldoor" );
    common_impl.PassingMgWin();
    cmn.GotoRoom( "rm_porch_ext" ); 

  end;

  cmn.MiniGameHide();
  cmn.Lock( true );

  common_impl.PlayAudio( "sfx", "levelext/sfx/aud_win_mghoteldoor", 0, "", 0 );

  ObjAnimate( "obj_hoteldoor_level_general", "alp",0,0, "", { 0, 0, 1,    0.6, 0, 0 } );

  for i = 1, HOTELDOOR_LEVEL_COUNT, 1 do 

    local curDigit = HOTELDOOR_DIGIT_NAME..i;
    local digitAlpha = ObjGet( curDigit ).alp;
    ObjAnimate( curDigit, "alp",0,0, "", { 0, 0, digitAlpha,    0.4, 0, 0 } );

    local curLamp = HOTELDOOR_LAMP_NAME..i; 
    local trg_after = "";

    if i == HOTELDOOR_LEVEL_COUNT then 
      trg_after = func_end;
    end; 

    ObjStopAnimate( curLamp, 8 );
    ObjSet( curLamp, { visible = true } );

    local lampAlpha = ObjGet( curLamp ).alp;
    ObjAnimate( curLamp, "alp",0,0, trg_after, { 0, 3, lampAlpha,    0.3, 3, 0,    0.8, 3, 1,    1.2, 3, 0.4,    2.1, 3, 1 } );

  end;

end;


--*********************************************************************************************************************
function public.Reset ()

  local gridCurY = ObjGet( "spr_hoteldoor_grid" ).pos_y; 
  ObjAnimate( "spr_hoteldoor_grid", "pos_y",0,0, "", { 0, 3, gridCurY,    1.2, 3, 900 } );

  local curDigit = HOTELDOOR_DIGIT_NAME..curLevel;
  local digitAlpha = ObjGet( curDigit ).alp;
  ObjAnimate( curDigit, "alp",0,0, "", { 0, 0, digitAlpha,    0.4, 0, 1 } );

  ng_global.progress[ "ext" ][ "win_hoteldoor" ].field = nil;
  ng_global.progress[ "ext" ][ "win_hoteldoor" ].removed_balls = nil;  
  private.Load();
  private.LevelChoosing(); 
  private.Update();
  
  common_impl.PlayAudio( "sfx", "level3/sfx/aud_single_clk_lamp", 0, "", 0 );

end;


--*********************************************************************************************************************
function public.Skip ()

port.skip_mg = true;
  private.Win();

end;








